import java.util.HashMap;
import java.util.Map;

import javax.jms.Connection;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;

import org.hornetq.api.core.TransportConfiguration;
import org.hornetq.api.jms.HornetQJMSClient;
import org.hornetq.api.jms.JMSFactoryType;
import org.hornetq.core.remoting.impl.netty.NettyConnectorFactory;
import org.hornetq.core.remoting.impl.netty.TransportConstants;
import org.hornetq.jms.client.HornetQConnectionFactory;

/**
 * Test to see if wildcard and source queues receive duplicate messages.
 * @author henry
 *
 */
public class TestJmsSender {
	
	private static int consumerCount = 0;
	private static int receivedCount = 0;
	
	public static final void main(String[] args) throws Exception {
		
		// Create a connection to the broker.
		Connection jmsConnection = createConnection();
		
		//
		// First a sanity check.
		//
		
		System.out.println("Running sanity test.");

		// Send a single message to the generic queue.
		sendMessage(jmsConnection, "job.generic");
		
		// Create two comsumers, for the same queue.
		receiveMessage(jmsConnection, "job.generic");
		receiveMessage(jmsConnection, "job.generic");
		
		// Wait for the consumers to terminate.
		while (consumerCount > 0) {
			try {
				Thread.sleep(1000L);
			} catch (InterruptedException ie) {
				// That's ok.
			}
		}
		
		if (receivedCount != 1) {
			System.out.println("Duplicate delivery for a single queue -- something's pretty badly amiss!");
			System.exit(1);
		} else {
			System.out.println("Sanity check ok, no duplicate messages received for two consumers on the same queue -- now running the real test.");
		}
		
		//
		// Now the real test.
		//
		consumerCount = 0;
		receivedCount = 0;
		
		// Send a single message to the generic queue.
		sendMessage(jmsConnection, "job.generic");
		
		// Create two consumers, one for the generic queue, one for a wildcard queue.
		receiveMessage(jmsConnection, "job.generic");
		receiveMessage(jmsConnection, "job.#");
		
		// Wait for the consumers to terminate.
		while (consumerCount > 0) {
			try {
				Thread.sleep(1000L);
			} catch (InterruptedException ie) {
				// That's ok.
			}
		}
		
		System.out.println("Number of messages processed: " + receivedCount + ", my (likely flawed) assumption had been that this would be 1!");
	}
	
	
	private static void sendMessage(Connection jmsConnection, String queueName) throws Exception {
		System.out.println("Sending message to queue: " + queueName);
		Session session = jmsConnection.createSession(false, Session.AUTO_ACKNOWLEDGE);
		Queue queue = HornetQJMSClient.createQueue(queueName);
		MessageProducer messageProducer = session.createProducer(queue);
		TextMessage message = session.createTextMessage("a test message");
		messageProducer.send(message);
	}
	
	private static void receiveMessage(final Connection jmsConnection, final String queueName) throws Exception {
		
		consumerStarted();
		
		Thread thread = new Thread(new Runnable() {

			@SuppressWarnings("synthetic-access")
			@Override
			public void run() {
				Message message = null;
				try {
					Queue queue = HornetQJMSClient.createQueue(queueName);
					Session session = jmsConnection.createSession(false, Session.CLIENT_ACKNOWLEDGE);
					MessageConsumer messageConsumer = session.createConsumer(queue);
					
					System.out.println("Waiting to receive message on queue: " + queueName);
					message = messageConsumer.receive(5000);

					if (message != null) {
						message.acknowledge();
					}
					messageConsumer.close();
					
				} catch (Throwable t) {
					t.printStackTrace();
				}
				
				if (message != null) {
					consumerStopped(1);
				} else {
					consumerStopped(0);
				}
			} 
		});
		thread.start();
	}
	
	private static Connection createConnection() throws Exception {
		
		Map<String, Object> server1Params = new HashMap<String, Object>();
		server1Params.put(TransportConstants.HOST_PROP_NAME, "localhost");
		server1Params.put(TransportConstants.PORT_PROP_NAME, "5445");
		TransportConfiguration server1Configuration = new TransportConfiguration(NettyConnectorFactory.class.getName(), server1Params);
		HornetQConnectionFactory hornetQConnectionFactory = HornetQJMSClient.createConnectionFactoryWithoutHA(JMSFactoryType.CF, server1Configuration);

		hornetQConnectionFactory.setRetryInterval(1000L);
		hornetQConnectionFactory.setRetryIntervalMultiplier(1.0);
		hornetQConnectionFactory.setReconnectAttempts(-1);
		hornetQConnectionFactory.setConnectionTTL(10000L);
		hornetQConnectionFactory.setClientFailureCheckPeriod(5000L);
		Connection jmsConnection = hornetQConnectionFactory.createConnection();
		jmsConnection.start();
		return jmsConnection;
	}
	
	private static synchronized void consumerStarted() {
		consumerCount++;
	}
	
	private static synchronized void consumerStopped(int messagesReceived) {
		consumerCount--;
		receivedCount += messagesReceived;
	}
}
